#pragma once

#include "Layout.h"
#include "Container.h"
#include "ScriptletContainerFactory.h"

class NPCOMMONUI_API CStackHorizontalLayoutManager : public ILayoutManager
{
public:
	DECLARE_PROPERTY_CONTAINER(CStackHorizontalLayoutManager);
	CStackHorizontalLayoutManager() {}
	virtual ~CStackHorizontalLayoutManager() {}

	//override
	//ILayoutManager
	void LayoutContainer(CContainer* pContainer)
	{
		CRect rcClient;		
		pContainer->GetDrawingBounds(&rcClient);		
		CRect rcPadding(0, 0, 0, 0);
		pContainer->GetPadding(&rcPadding);
		CAtlList<CComponent*> vComponent;
		pContainer->GetChildList(vComponent);

		CRect rcComponent;
		CSize sizeComponentPreferred;
		int nPosX = rcPadding.left;
		int nPosY = rcPadding.top;
		CRect rcMargin(0, 0, 0, 0);
		HorizonAlignment::eHorizonAlignment horizontalAlignment = HorizonAlignment::Stretch;
		VerticalAlignment::eVerticalAlignment verticalAlignment = VerticalAlignment::Stretch;

		POSITION pos = vComponent.GetHeadPosition();
		while (pos)
		{
			CComponent* pComponent = vComponent.GetNext(pos);
			if (pComponent->IsVisible() == FALSE) continue;
			
			sizeComponentPreferred.SetSize(0, 0);
			pComponent->GetPreferredSize(&sizeComponentPreferred);
			pComponent->GetMargin(&rcMargin);

			horizontalAlignment = pComponent->GetHorizontalAligment();
			verticalAlignment = pComponent->GetVerticalAlignment();
			
			rcComponent.left = nPosX + rcMargin.left;
			rcComponent.right = rcComponent.left + sizeComponentPreferred.cx;

			if (verticalAlignment == VerticalAlignment.Stretch)
			{
				rcComponent.top = nPosY + rcMargin.top;
				rcComponent.bottom = rcClient.Height() - rcMargin.bottom;
			}
			else if (verticalAlignment == VerticalAlignment.Center)
			{
				rcComponent.top = nPosY + (rcClient.Height()-sizeComponentPreferred.cy)/2 + rcMargin.top;
				rcComponent.bottom = rcComponent.top + sizeComponentPreferred.cy;
			}
			else if (verticalAlignment == VerticalAlignment.Bottom)
			{
				rcComponent.top = nPosY+ rcClient.Height() - sizeComponentPreferred.cy + rcMargin.top;
				rcComponent.bottom = rcComponent.top + sizeComponentPreferred.cy;
			}
			else
			{
				rcComponent.top = nPosY + rcMargin.top;
				rcComponent.bottom = rcComponent.top + sizeComponentPreferred.cy;
			}
			if (NULL == pos && horizontalAlignment == HorizonAlignment.Stretch)
			{
				rcComponent.right = max(rcComponent.right, rcClient.Width());
			}

			pComponent->SetActualBound(rcComponent.left, rcComponent.top, rcComponent.Width(), rcComponent.Height());

			nPosX += sizeComponentPreferred.cx + rcMargin.left + rcMargin.right;
		}
	}
};

class NPCOMMONUI_API CStackVerticalLayoutManager : public ILayoutManager
{
public:
	DECLARE_PROPERTY_CONTAINER(CStackVerticalLayoutManager);
	CStackVerticalLayoutManager() {}
	virtual ~CStackVerticalLayoutManager() {}

	//override
	//ILayoutManager
	void LayoutContainer(CContainer* pContainer)
	{
		CRect rcClient;		
		pContainer->GetDrawingBounds(&rcClient);
		CRect rcPadding(0, 0, 0, 0);
		pContainer->GetPadding(&rcPadding);

		CAtlList<CComponent*> vComponent;
		pContainer->GetChildList(vComponent);

		CRect rcComponent;
		CSize sizeComponentPreferred;
		int nPosX = rcPadding.left;
		int nPosY = rcPadding.top;
		CRect rcMargin(0, 0, 0, 0);
		HorizonAlignment::eHorizonAlignment horizontalAlignment = HorizonAlignment::Stretch;
		VerticalAlignment::eVerticalAlignment verticalAlignment = VerticalAlignment::Stretch;

		POSITION pos = vComponent.GetHeadPosition();
		while (pos)
		{
			CComponent* pComponent = vComponent.GetNext(pos);
			if (pComponent->IsVisible() == FALSE) continue;

			sizeComponentPreferred.SetSize(0, 0);
			pComponent->GetPreferredSize(&sizeComponentPreferred);
			pComponent->GetMargin(&rcMargin);
			
			horizontalAlignment = pComponent->GetHorizontalAligment();
			verticalAlignment = pComponent->GetVerticalAlignment();
			
			rcComponent.top = nPosY + rcMargin.top;
			rcComponent.bottom = rcComponent.top + sizeComponentPreferred.cy;

			if (horizontalAlignment == HorizonAlignment.Stretch)
			{
				rcComponent.left = nPosX + rcMargin.left;
				rcComponent.right = rcClient.Width() - rcMargin.right;
			}
			else if (horizontalAlignment == HorizonAlignment.Center)
			{
				rcComponent.left = nPosX + (rcClient.Width()-sizeComponentPreferred.cx)/2 + rcMargin.left;
				rcComponent.right = rcComponent.left + sizeComponentPreferred.cx;
			}
			else if (horizontalAlignment == HorizonAlignment.Right)
			{
				rcComponent.left = nPosX + rcClient.Width() - sizeComponentPreferred.cx + rcMargin.left;
				rcComponent.right = rcComponent.left + sizeComponentPreferred.cx;
			}
			else
			{
				rcComponent.left = nPosX + rcMargin.left;
				rcComponent.right = rcComponent.left + sizeComponentPreferred.cx;
			}
			 
			if (NULL == pos && verticalAlignment == VerticalAlignment.Stretch)
			{
				rcComponent.bottom = max(rcComponent.bottom, rcClient.Height());
			}

			pComponent->SetActualBound(rcComponent.left, rcComponent.top, rcComponent.Width(), rcComponent.Height());

			nPosY += sizeComponentPreferred.cy + rcMargin.top + rcMargin.bottom;
		}
	}
};